var Demux = function()
{
	var demux = {};
	var debug = 1;
	
	var subtypeSequencingRun = null;
	var clusterIsValid = false;
	
	// Page initialization
	demux.initPage = function()
	{
		
		// Step 1
		Buttons.addClickHandler('btnSelectSeqRuns', demux.selectSeqencingRuns);
		Events.addEventHandler('seqRuns', 'base-selected', demux.setSeqRunCallback);
		Events.addEventHandler('seqRuns', 'change', demux.seqRunsOnChange);
		Events.addEventHandler('step-1', 'wizard-validate', demux.validateStep1);

		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', demux.initializeStep2);
		Events.addEventHandler('step-2', 'wizard-validate', demux.validateStep2);
		Events.addEventHandler('clusters', 'change', demux.clusterOnChange);
		
		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
			
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', demux.submit);
		

		Wizard.showLoadingAnimation('Loading bioassays...');
		var url = '../DemuxMerge.servlet?ID='+App.getSessionId();
		url += '&cmd=GetConfirmedSequencingRuns';
		Wizard.asyncJsonRequest(url, demux.initializeStep1);
	}
	
	// --- Step 1 -----------------------------------
	demux.initializeStep1 = function(response)
	{
		var seqRuns = response.sequencingRuns;
		seqRuns.sort(demux.sortByPool);
		var frm = document.forms['reggie'];	

		if (seqRuns.length > 0)
		{
			for (var runNo=0; runNo < seqRuns.length; runNo++)
			{
				var seqRun = seqRuns[runNo];
				var option = demux.getOptionForSeqRun(seqRun);
				frm.seqRuns.options[frm.seqRuns.length] = option;
			}
		}
		else
		{
			Wizard.setInputStatus('seqRuns', 'invalid', 'No new sequencing runs available for processing');
		}
		
		Doc.show('step-1');
		Doc.show('gonext');

		frm.seqRuns.focus();
	}
	
	demux.validateStep1 = function(event)
	{
		var frm = document.forms['reggie'];
		for (var seqNo = 0; seqNo < frm.seqRuns.length; seqNo++)
		{
			if (frm.seqRuns[seqNo].selected) return;
		}
		Wizard.setInputStatus('seqRuns', 'invalid', 'Select at least one sequencing run');
		event.preventDefault();
	}
	
	/**
		Sort by the name of the first pool.
	*/
	demux.sortByPool = function(a, b)
	{
		var p1 = a.flowCell.pools[0].name;
		var p2 = b.flowCell.pools[0].name;
		if (p1 < p2) return -1;
		if (p1 > p2) return 1;
		if (a.name < b.name) return -1;
		if (a.name > b.name) return 1;
		return a.id - b.id;
	}
	
	demux.getOptionForSeqRun = function(seqRun)
	{
		var flowCell = seqRun.flowCell;
		var name = seqRun.name+': ';
		var numPools = flowCell.pools.length;
		var firstPoolNum = demux.getPoolNum(flowCell.pools[0].name);
		var lastPoolNum = demux.getPoolNum(flowCell.pools[flowCell.pools.length-1].name);

		if (lastPoolNum - firstPoolNum == numPools - 1)
		{
			if (numPools > 1)
			{
				// Display: PoolN -- PoolY
				name += flowCell.pools[0].name + ' — ' + flowCell.pools[flowCell.pools.length-1].name;
			}
			else
			{
				name += flowCell.pools[0].name;
			}
		}
		else
		{
			// Display: PoolN + x more...
			name += flowCell.pools[0].name + ' + ' + (numPools-1) + ' more...';
		}

		// Tooltip is always all pools
		var title = '';
		for (var poolNo=0; poolNo < numPools; poolNo++)
		{
			var pool = flowCell.pools[poolNo];
			if (poolNo > 0) title += ', ';
			title += pool.name;
		}

		var option = new Option(name, seqRun.id);
		option.title = title;
		option.seqRun = seqRun;	
		return option;
	}

	demux.getPoolNum = function(poolName)
	{
		var num = poolName.match(/Pool(\d+)/);
		return num ? parseInt(num[1], 10) : null;
	}

	demux.selectSeqencingRuns = function()
	{
		var frm = document.forms['reggie'];
		if (frm.seqRuns.disabled) return;
		
		if (subtypeSequencingRun == null)
		{
			subtypeSequencingRun = Reggie.getSubtypeInfo('SEQUENCING_RUN');
		}
		
		var url = '&resetTemporary=1';
		url += '&tmpfilter:INT:itemSubtype='+subtypeSequencingRun.id;
		Dialogs.selectItem('DERIVEDBIOASSAY', 'seqRuns', 1, url);
	}

	demux.setSeqRunCallback = function(event)
	{
		var frm = document.forms['reggie'];
		var seqRunList = frm.seqRuns;
		for (var i = 0; i < seqRunList.length; i++)
		{
			if (seqRunList[i].value == event.detail.id)
			{
				seqRunList[i].selected = true;
				demux.seqRunsOnChange();
				return;
			}
		}
		
		// Get more information about the selected sequencing run
		var url = '../DemuxMerge.servlet?ID='+App.getSessionId();
		url += '&cmd=GetSequencingRunInfo&id=' + event.detail.id;
		Wizard.asyncJsonRequest(url, demux.seqRunInfoLoaded);
	}
	
	demux.seqRunInfoLoaded = function(response)
	{
		var seqRun = response.sequencingRun;
		var option = demux.getOptionForSeqRun(seqRun);
		option.selected = true;
		var frm = document.forms['reggie'];
		var seqRunList = frm.seqRuns;
		seqRunList[seqRunList.length] = option;
		demux.seqRunsOnChange();
	}

	// Check if all selected sequencing runs have the same pools
	demux.seqRunsOnChange = function()
	{
		var frm = document.forms['reggie'];
		var numSelected = 0;
		var pools;
		Wizard.setInputStatus('seqRuns');
		for (var seqNo = 0; seqNo < frm.seqRuns.length; seqNo++)
		{
			if (frm.seqRuns[seqNo].selected)
			{
				numSelected ++;
				if (numSelected == 1)
				{
					pools = frm.seqRuns[seqNo].seqRun.flowCell.pools;
				}
				else
				{
					if (!demux.compareListOfPools(pools, frm.seqRuns[seqNo].seqRun.flowCell.pools))
					{
						Wizard.setInputStatus('seqRuns', 'warning', 'The selected sequencing runs have different pools');
						return;
					}
				}
			}
		}
		
		if (numSelected == 0)
		{
			Wizard.setInputStatus('seqRuns', 'invalid', 'Select at least one sequencing run');
		}
		else
		{
			Wizard.setInputStatus('seqRuns', 'valid');
		}
	}
	
	// Check if two lists of pools are the same or not
	demux.compareListOfPools = function(p1, p2)
	{
		if (p1.length != p2.length) return false;
		for (var i = 0; i < p1.length; i++)
		{
			if (p1[i].id != p2[i].id) return false;
		}
		return true;
	}
	
	demux.initializeStep2 = function()
	{
		var frm = document.forms['reggie'];
		
		// ReadString input fields
		var html = '';
		for (var seqNo = 0; seqNo < frm.seqRuns.length; seqNo++)
		{
			if (frm.seqRuns[seqNo].selected)
			{
				var seqRun = frm.seqRuns[seqNo].seqRun;
				var fc = seqRun.flowCell;
				var readStringId = 'readString.'+seqNo;
				html += '<tr>';
				html += '<td class="prompt">'+Strings.encodeTags(seqRun.name)+'</td>';
				html += '<td>'+Strings.encodeTags(seqRun.SequencingCycles)+'</td>';
				html += '<td class="status"></td>';
				html += '<td class="help"></td>';
				html += '</tr>';
				
				html += '<tr>';
				html += '<td class="subprompt">ReadString</td>';
				html += '<td><input type="text" class="required" name="'+readStringId+'" id="'+readStringId+'" value="'+Strings.encodeTags(seqRun.ReadString)+'" style="width: 11em;"></td>';
				html += '<td class="status" id="'+readStringId+'.status"></td>';
				html += '<td class="help"><span id="'+readStringId+'.message"></span></td>';
				html += '</tr>';
				
				html += '<tr>';
				html += '<td class="subprompt">Omit lanes</td>';
				html += '<td>';
				for (var laneNo = 1; laneNo <= fc.numLanes; laneNo++)
				{
					var checked = fc.FailedLanes.indexOf(laneNo) >= 0 ? 'checked' : '';
					html += '<label><input type="checkbox" name="omitLanes.'+seqNo+'" value="'+laneNo+'" ' + checked + '>'+laneNo+'</label> ';
				}
				html += '</td>';
				html += '<td class="status"></td>';
				html += '<td class="help">By default, failed lanes are omitted</td>';
				html += '</tr>';
				
			}
		}
		Doc.element('SequencingRun-section').innerHTML = html;
		for (var seqNo = 0; seqNo < frm.seqRuns.length; seqNo++)
		{
			if (frm.seqRuns[seqNo].selected)
			{
				demux.readStringOnBlur(frm['readString.'+seqNo]);
				Events.addEventHandler('readString.'+seqNo, 'blur', demux.readStringOnBlur);
			}
		}
		
		Wizard.setCurrentStep(2);
		Doc.show('gocancel');
		Doc.show('goregister');
		
		// Check debug by default if debug is set or not on a secure server (=production server)
		frm.debug.checked = debug || location.protocol != 'https:';

		Reggie.loadProtocols('DEMUX_PROTOCOL', 'demuxProtocol');
		Reggie.loadSoftware('DEMUX_SOFTWARE', 'demuxSoftware');
		Reggie.loadProtocols('MERGE_PROTOCOL', 'mergeProtocol');
		Reggie.loadSoftware('MERGE_SOFTWARE', 'mergeSoftware');

		// Load open grid clusters
		var url = '../OpenGrid.servlet?ID='+App.getSessionId() + '&cmd=GetHostInfo';
		Doc.addClass('clusters', 'list-loading');
		frm.clusters[0] = new Option('loading...');
		Wizard.asyncJsonRequest(url, demux.clustersLoaded);
	}

	demux.clustersLoaded = function(response)
	{
		Doc.removeClass('clusters', 'list-loading');
		var frm = document.forms['reggie'];
		var clusters = response.clusters;
		frm.clusters.length = 0;
		for (var i = 0; i < clusters.length; i++)
		{
			var cl = clusters[i];
			var option = new Option(cl.name, cl.id);
			option.cluster = cl;
			frm.clusters[frm.clusters.length] = option;
			Wizard.setInputStatus('clusters', 'valid');
			clusterIsValid = true;
		}
		if (frm.clusters.length == 0)
		{
			Wizard.setInputStatus('clusters', 'invalid', 'No available Open Grid clusters');
			clusterIsValid = false;
		}
		else
		{
			demux.clusterOnChange();
		}
	}

	demux.clusterOnChange = function()
	{
		var frm = document.forms['reggie'];
		var cluster = frm.clusters[frm.clusters.selectedIndex].cluster;
		if (cluster.priorities && cluster.priorities.length > 0)
		{
			frm.priority.length = 0;
			for (var pNo = 0; pNo < cluster.priorities.length; pNo++)
			{
				var p = cluster.priorities[pNo];
				frm.priority[frm.priority.length] = new Option(p.name + ' ('+p.value+')', p.value, p['default']);
			}
			Doc.show('job-priority');
		}
		else
		{
			Doc.hide('job-priority');
		}
	}

	demux.readStringOnBlur = function(eventOrTarget)
	{
		var target = eventOrTarget.currentTarget || eventOrTarget;
		var isValid = false;

		Wizard.setInputStatus(target.id);
		var readString = target.value;
		
		if (!readString.match(/^(\d+[TBS])+$/))
		{
			Wizard.setInputStatus(target.id, 'invalid', 'Invalid read string');
		}
		else
		{
			isValid = true;
			Wizard.setInputStatus(target.id, 'valid');
		}
		
		Data.set(target, 'valid', isValid ? 1 : 0);
	}

	demux.validateStep2 = function(event)
	{
		
		if (!clusterIsValid)
		{
			event.preventDefault();
			return;
		}
			
		var frm = document.forms['reggie'];
		for (var seqNo = 0; seqNo < frm.seqRuns.length; seqNo++)
		{
			if (frm.seqRuns[seqNo].selected) 
			{
				if (!Wizard.isValid(frm['readString.'+seqNo]))
				{
					event.preventDefault();
					return;
				}
			}
		}
	}
	
	demux.submit = function()
	{
		var frm = document.forms['reggie'];
		var submitInfo = {};
		submitInfo.demuxSoftware = parseInt(frm.demuxSoftware.value);
		submitInfo.demuxProtocol = parseInt(frm.demuxProtocol.value);
		submitInfo.mergeSoftware = parseInt(frm.mergeSoftware.value);
		submitInfo.mergeProtocol = parseInt(frm.mergeProtocol.value);
		submitInfo.cluster = frm.clusters.value;
		if (frm.priority.selectedIndex >= 0)
		{
			submitInfo.priority = parseInt(frm.priority.value);
		}
		submitInfo.debug = frm.debug.checked;
		
		var sequencingRuns = [];
		submitInfo.sequencingRuns = sequencingRuns;
		
		for (var seqNo = 0; seqNo < frm.seqRuns.length; seqNo++)
		{
			if (frm.seqRuns[seqNo].selected) 
			{
				var seqRun = {};
				seqRun.id = frm.seqRuns[seqNo].seqRun.id;
				seqRun.ReadString = frm['readString.'+seqNo].value;
				
				var omitLanes = [];
				var checkboxes = frm['omitLanes.'+seqNo];
				for (var i = 0; i < checkboxes.length; i++)
				{
					if (checkboxes[i].checked) 
					{
						omitLanes[omitLanes.length] = parseInt(checkboxes[i].value);
					}
				}
				seqRun.OmitLanes = omitLanes;
				sequencingRuns[sequencingRuns.length] = seqRun;
			}
		}
		
		var url = '../DemuxMerge.servlet?ID='+App.getSessionId();
		url += '&cmd=StartDemuxAndMerge';
		
		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, demux.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	demux.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}

	return demux;
}();

Doc.onLoad(Demux.initPage);

